@@ -26,7 +26,8 @@ from curses_misc import TextDialog,DynWrap,MaskingEdit,ComboBox,error
 import wicd.misc as misc
 from wicd.misc import noneToString, stringToNone, noneToBlankString, to_bool
 
-from wicd.translations import language
+from wicd.translations import language, _
+import os
 
 daemon = None
 wired = None
@@ -45,27 +46,30 @@ class AdvancedSettingsDialog(urwid.WidgetWrap):
     def __init__(self):
         self.ui=None
 
-        static_ip_t = language['use_static_ip']
-        ip_t        = ('editcp',language['ip']+':     ')
-        netmask_t   = ('editcp',language['netmask']+':')
-        gateway_t   = ('editcp',language['gateway']+':')
+        static_ip_t = _('Use Static IPs')
+        ip_t        = ('editcp',_('IP')+':     ')
+        netmask_t   = ('editcp',_('Netmask')+':')
+        gateway_t   = ('editcp',_('Gateway')+':')
 
-        use_static_dns_t = language['use_static_dns']
-        use_global_dns_t = language['use_global_dns']
-        dns_dom_t    = ('editcp',language['dns_domain']+':   ')
-        search_dom_t = ('editcp',language['search_domain']+':')
-        dns1_t       = ('editcp',language['dns']+ ' 1'+':'+' '*8)
-        dns2_t       = ('editcp',language['dns']+ ' 2'+':'+' '*8)
-        dns3_t       = ('editcp',language['dns']+ ' 3'+':'+' '*8)
-
-        cancel_t = 'Cancel'
-        ok_t = 'OK'
+        use_static_dns_t = _('Use Static DNS')
+        use_global_dns_t = _('Use global DNS servers')
+        dns_dom_t    = ('editcp',_('DNS domain')+':   ')
+        search_dom_t = ('editcp',_('Search domain')+':')
+        dns1_t       = ('editcp',_('DNS server')+ ' 1'+':'+' '*8)
+        dns2_t       = ('editcp',_('DNS server')+ ' 2'+':'+' '*8)
+        dns3_t       = ('editcp',_('DNS server')+ ' 3'+':'+' '*8)
+        
+        use_dhcp_h_t = _('Use DHCP Hostname')
+        dhcp_h_t     = ('editcp',_('DHCP Hostname')+': ')
+        
+        cancel_t = _('Cancel')
+        ok_t = _('OK')
         
         self.static_ip_cb = urwid.CheckBox(static_ip_t,
                 on_state_change=self.static_ip_toggle)
-        self.ip_edit     =DynWrap(urwid.Edit(ip_t),False)
-        self.netmask_edit=DynWrap(urwid.Edit(netmask_t),False)
-        self.gateway_edit=DynWrap(urwid.Edit(gateway_t),False)
+        self.ip_edit      = DynWrap(urwid.Edit(ip_t),False)
+        self.netmask_edit = DynWrap(urwid.Edit(netmask_t),False)
+        self.gateway_edit = DynWrap(urwid.Edit(gateway_t),False)
 
 
         self.static_dns_cb = DynWrap(urwid.CheckBox(use_static_dns_t,
@@ -80,6 +84,9 @@ class AdvancedSettingsDialog(urwid.WidgetWrap):
         self.dns2              = DynWrap(urwid.Edit(dns2_t)      ,False)
         self.dns3              = DynWrap(urwid.Edit(dns3_t)      ,False)
 
+        self.use_dhcp_h        = urwid.CheckBox(use_dhcp_h_t,False,on_state_change=self.use_dhcp_h_toggle)
+        self.dhcp_h            = DynWrap(urwid.Edit(dhcp_h_t),False)
+
         _blank = urwid.Text('')
 
         walker = urwid.SimpleListWalker([self.static_ip_cb,
@@ -89,16 +96,22 @@ class AdvancedSettingsDialog(urwid.WidgetWrap):
                                          _blank,
                                          self.checkb_cols,
                                          self.dns_dom_edit,self.search_dom_edit,
-                                         self.dns1,self.dns2,self.dns3
+                                         self.dns1,self.dns2,self.dns3,
+                                         _blank,
+                                         self.use_dhcp_h,
+                                         self.dhcp_h,
+                                         _blank
                                         ])
 
 
 
         self._listbox = urwid.ListBox(walker)
-        #self._frame = urwid.Frame(self._listbox)
         self._frame = urwid.Frame(self._listbox)
         self.__super.__init__(self._frame)
     
+    def use_dhcp_h_toggle(self,checkb,new_state,user_data=None):
+        self.dhcp_h.set_sensitive(new_state)
+
     def static_ip_toggle(self,checkb,new_state,user_data=None):
         for w in [ self.ip_edit,self.netmask_edit,self.gateway_edit ]:
             w.set_sensitive(new_state)
@@ -128,6 +141,9 @@ class AdvancedSettingsDialog(urwid.WidgetWrap):
     def save_settings(self):
         """ Save settings common to wired and wireless settings dialogs. """
         if self.static_ip_cb.get_state():
+            for i in [self.ip_edit,self.netmask_edit,self.gateway_edit]:
+                i.set_edit_text(i.get_edit_text().strip())
+
             self.set_net_prop("ip", noneToString(self.ip_edit.get_edit_text()))
             self.set_net_prop("netmask", noneToString(self.netmask_edit.get_edit_text()))
             self.set_net_prop("gateway", noneToString(self.gateway_edit.get_edit_text()))
@@ -140,6 +156,10 @@ class AdvancedSettingsDialog(urwid.WidgetWrap):
            not self.global_dns_cb.get_state():
             self.set_net_prop('use_static_dns', True)
             self.set_net_prop('use_global_dns', False)
+            # Strip addressses before checking them in the daemon.
+            for i in [self.dns1, self.dns2,
+                      self.dns3,self.dns_dom_edit, self.search_dom_edit]:
+                i.set_edit_text(i.get_edit_text().strip())
             self.set_net_prop('dns_domain', noneToString(self.dns_dom_edit.get_edit_text()))
             self.set_net_prop("search_domain", noneToString(self.search_dom_edit.get_edit_text()))
             self.set_net_prop("dns1", noneToString(self.dns1.get_edit_text()))
@@ -157,21 +177,90 @@ class AdvancedSettingsDialog(urwid.WidgetWrap):
             self.set_net_prop("dns1", '')
             self.set_net_prop("dns2", '')
             self.set_net_prop("dns3", '')
+        self.set_net_prop('dhcphostname',self.dhcp_h.get_edit_text())
+        self.set_net_prop('usedhcphostname',self.use_dhcp_h.get_state())
     # Prevent comboboxes from dying.
     def ready_widgets(self,ui,body):
         pass
 
+    def combo_on_change(self,combobox,new_index,user_data=None):
+        self.change_encrypt_method()
+    
+    # More or less ripped from netentry.py
+    def change_encrypt_method(self):
+        #self.lbox_encrypt = urwid.ListBox()
+        self.encryption_info = {}
+        wid,ID = self.encryption_combo.get_focus()
+        methods = self.encrypt_types
+
+        if self._w.body.body.__contains__(self.pile_encrypt):
+            self._w.body.body.pop(self._w.body.body.__len__()-1)
+
+        # If nothing is selected, select the first entry.
+        if ID == -1:
+            self.encryption_combo.set_focus(0)
+            ID = 0
+
+        theList = []
+        for type_ in ['required', 'optional']:
+            fields = methods[ID][type_]
+            for field in fields:
+                try:
+                    edit = MaskingEdit(('editcp',language[field[1].lower().replace(' ','_')]+': '))
+                except KeyError:
+                    edit = MaskingEdit(('editcp',field[1].replace(' ','_')+': '))
+                edit.set_mask_mode('no_focus')
+                theList.append(edit)
+                # Add the data to any array, so that the information
+                # can be easily accessed by giving the name of the wanted
+                # data.
+                self.encryption_info[field[0]] = [edit, type_]
+                
+                if self.wired:
+                    edit.set_edit_text(noneToBlankString(
+                        wired.GetWiredProperty(field[0])))
+                else:
+                    edit.set_edit_text(noneToBlankString(
+                        wireless.GetWirelessProperty(self.networkid, field[0])))
+
+        #FIXME: This causes the entire pile to light up upon use.
+        # Make this into a listbox?
+        self.pile_encrypt = DynWrap(urwid.Pile(theList),attrs=('editbx','editnfc'))
+        
+        self.pile_encrypt.set_sensitive(self.encryption_chkbox.get_state())
+        
+        self._w.body.body.insert(self._w.body.body.__len__(),self.pile_encrypt)
+        #self._w.body.body.append(self.pile_encrypt)
+    
+    def encryption_toggle(self,chkbox,new_state,user_data=None):
+        self.encryption_combo.set_sensitive(new_state)
+        self.pile_encrypt.set_sensitive(new_state)
+
 class WiredSettingsDialog(AdvancedSettingsDialog):
-    def __init__(self,name):
+    def __init__(self,name,parent):
         global wired, daemon
         AdvancedSettingsDialog.__init__(self)
-        self.set_default = urwid.CheckBox(language['default_wired'])
+        self.wired = True
+        
+        self.set_default = urwid.CheckBox(_('Use as default profile (overwrites any previous default)'))
         #self.cur_default = 
         # Add widgets to listbox
         self._w.body.body.append(self.set_default)
         
+        self.parent = parent
+        encryption_t = _('Use Encryption')
+        
+        self.encryption_chkbox = urwid.CheckBox(encryption_t,on_state_change=self.encryption_toggle)
+        self.encryption_combo = ComboBox(callback=self.combo_on_change)
+        self.pile_encrypt = None
+        # _w is a Frame, _w.body is a ListBox, _w.body.body is the ListWalker :-)
+        self._listbox.body.append(self.encryption_chkbox)
+        self._listbox.body.append(self.encryption_combo)
+        self.encrypt_types = misc.LoadEncryptionMethods(wired = True)
+        self.set_values()
+        
         self.prof_name = name
-        title = language['configuring_wired'].replace('$A',self.prof_name)
+        title = _('Configuring preferences for wired profile "$A"').replace('$A',self.prof_name)
         self._w.header = urwid.Text( ('header',title),align='right' )
 
         self.set_values()
@@ -196,11 +285,62 @@ class WiredSettingsDialog(AdvancedSettingsDialog):
 
         self.set_default.set_state(to_bool(wired.GetWiredProperty("default")))
 
+        # Throw the encryption stuff into a list
+        list = []
+        activeID = -1  # Set the menu to this item when we are done
+        for x, enc_type in enumerate(self.encrypt_types):
+            list.append(enc_type['name'])
+            if enc_type['type'] == wired.GetWiredProperty("enctype"):
+                activeID = x
+        self.encryption_combo.set_list(list)
+
+        self.encryption_combo.set_focus(activeID)
+        if wired.GetWiredProperty("encryption_enabled"):
+            self.encryption_chkbox.set_state(True,do_callback=False)
+            self.encryption_combo.set_sensitive(True)
+            #self.lbox_encrypt_info.set_sensitive(True)
+        else:
+            self.encryption_combo.set_focus(0)
+            self.encryption_combo.set_sensitive(False)
+
+        self.change_encrypt_method()
+
+        dhcphname = wired.GetWiredProperty("dhcphostname")
+        if dhcphname is None:
+            dhcphname = os.uname()[1]
+
+        self.use_dhcp_h.set_state(bool(wired.GetWiredProperty('usedhcphostname')))
+        self.dhcp_h.set_sensitive(self.use_dhcp_h.get_state())
+        self.dhcp_h.set_edit_text(unicode(dhcphname))
+
     def save_settings(self):
+        # Check encryption info
+        if self.encryption_chkbox.get_state():
+            encrypt_info = self.encryption_info
+            encrypt_methods = self.encrypt_types
+            self.set_net_prop("enctype",
+                               encrypt_methods[self.encryption_combo.get_focus()[1] ]['type'])
+            self.set_net_prop("encryption_enabled", True)
+            # Make sure all required fields are filled in.
+            for entry_info in encrypt_info.itervalues():
+                if entry_info[0].get_edit_text() == "" \
+                    and entry_info[1] == 'required':
+                    error(self.ui, self.parent,"%s (%s)" \
+                            % (_('Required encryption information is missing.'),
+                                entry_info[0].get_caption()[0:-2] )
+                          )
+                    return False
+
+            for entry_key, entry_info in encrypt_info.iteritems():
+                self.set_net_prop(entry_key, noneToString(entry_info[0].
+                                                   get_edit_text()))
+        else:
+            self.set_net_prop("enctype", "None")
+            self.set_net_prop("encryption_enabled", False)
+        
         AdvancedSettingsDialog.save_settings(self)
         if self.set_default.get_state():
             wired.UnsetWiredDefault()
-        print self.set_default.get_state()
         if self.set_default.get_state():
             bool = True
         else:
@@ -221,11 +361,13 @@ class WirelessSettingsDialog(AdvancedSettingsDialog):
     def __init__(self,networkID,parent):
         global wireless, daemon
         AdvancedSettingsDialog.__init__(self)
+        self.wired = False
+        
         self.networkid = networkID
         self.parent = parent
-        global_settings_t = language['global_settings']
-        encryption_t = language['use_encryption']
-        autoconnect_t = language['automatic_connect']
+        global_settings_t = _('Use these settings for all networks sharing this essid')
+        encryption_t = _('Use Encryption')
+        autoconnect_t = _('Automatically connect to this network')
         
         self.global_settings_chkbox = urwid.CheckBox(global_settings_t)
         self.encryption_chkbox = urwid.CheckBox(encryption_t,on_state_change=self.encryption_toggle)
@@ -233,7 +375,6 @@ class WirelessSettingsDialog(AdvancedSettingsDialog):
         self.autoconnect_chkbox = urwid.CheckBox(autoconnect_t)
         self.pile_encrypt = None
         # _w is a Frame, _w.body is a ListBox, _w.body.body is the ListWalker :-)
-        self._listbox.body.append(urwid.Text(""))
         self._listbox.body.append(self.global_settings_chkbox)
         self._listbox.body.append(self.autoconnect_chkbox)
         self._listbox.body.append(self.encryption_chkbox)
@@ -241,15 +382,8 @@ class WirelessSettingsDialog(AdvancedSettingsDialog):
         self.encrypt_types = misc.LoadEncryptionMethods()
         self.set_values()
 
-        title = language['configuring_wireless'].replace('$A',wireless.GetWirelessProperty(networkID,'essid')).replace('$B',wireless.GetWirelessProperty(networkID,'bssid'))
+        title = _('Configuring preferences for wireless network "$A" ($B)').replace('$A',wireless.GetWirelessProperty(networkID,'essid')).replace('$B',wireless.GetWirelessProperty(networkID,'bssid'))
         self._w.header = urwid.Text(('header',title),align='right' )
-    
-    def encryption_toggle(self,chkbox,new_state,user_data=None):
-        self.encryption_combo.set_sensitive(new_state)
-        self.pile_encrypt.set_sensitive(new_state)
-
-    def combo_on_change(self,combobox,new_index,user_data=None):
-        self.change_encrypt_method()
 
     def set_values(self):
         """ Set the various network settings to the right values. """
@@ -297,6 +431,12 @@ class WirelessSettingsDialog(AdvancedSettingsDialog):
             self.encryption_combo.set_focus(0)
 
         self.change_encrypt_method()
+        dhcphname = wireless.GetWirelessProperty(networkID,"dhcphostname")
+        if dhcphname is None:
+            dhcphname = os.uname()[1]
+        self.use_dhcp_h.set_state(bool(wireless.GetWirelessProperty(networkID,'usedhcphostname')))
+        self.dhcp_h.set_sensitive(self.use_dhcp_h.get_state())
+        self.dhcp_h.set_edit_text(unicode(dhcphname))
         
 
     def set_net_prop(self, option, value):
@@ -311,7 +451,6 @@ class WirelessSettingsDialog(AdvancedSettingsDialog):
     def save_settings(self):
         # Check encryption info
         if self.encryption_chkbox.get_state():
-            #print "setting encryption info..."
             encrypt_info = self.encryption_info
             encrypt_methods = self.encrypt_types
             self.set_net_prop("enctype",
@@ -321,7 +460,7 @@ class WirelessSettingsDialog(AdvancedSettingsDialog):
                 if entry_info[0].get_edit_text() == "" \
                     and entry_info[1] == 'required':
                     error(self.ui, self.parent,"%s (%s)" \
-                            % (language['encrypt_info_missing'], 
+                            % (_('Required encryption information is missing.'),
                                 entry_info[0].get_caption()[0:-2] )
                           )
                     return False
@@ -332,7 +471,7 @@ class WirelessSettingsDialog(AdvancedSettingsDialog):
         elif not self.encryption_chkbox.get_state() and \
              wireless.GetWirelessProperty(self.networkid, "encryption"):
             # Encrypt checkbox is off, but the network needs it.
-            error(self.ui, self.parent, language['enable_encryption'])
+            error(self.ui, self.parent, _('This network requires encryption to be enabled.'))
             return False
         else:
             self.set_net_prop("enctype", "None")
@@ -351,45 +490,6 @@ class WirelessSettingsDialog(AdvancedSettingsDialog):
         wireless.SaveWirelessNetworkProfile(self.networkid)
         return True
 
-    # More or less ripped from netentry.py
-    def change_encrypt_method(self):
-        #self.lbox_encrypt = urwid.ListBox()
-        self.encryption_info = {}
-        wid,ID = self.encryption_combo.get_focus()
-        methods = misc.LoadEncryptionMethods()
-
-        if self._w.body.body.__contains__(self.pile_encrypt):
-            self._w.body.body.pop(self._w.body.body.__len__()-1)
-
-        # If nothing is selected, select the first entry.
-        if ID == -1:
-            self.encryption_combo.set_focus(0)
-            ID = 0
-
-        theList = []
-        for type_ in ['required', 'optional']:
-            fields = methods[ID][type_]
-            for field in fields:
-                if language.has_key(field[1]):
-                    edit = MaskingEdit(('editcp',language[field[1].lower().replace(' ','_')]+': '))
-                else:
-                    edit = MaskingEdit(('editcp',field[1].replace('_',' ')+': '))
-                edit.set_mask_mode('no_focus')
-                theList.append(edit)
-                # Add the data to any array, so that the information
-                # can be easily accessed by giving the name of the wanted
-                # data.
-                self.encryption_info[field[0]] = [edit, type_]
-
-                edit.set_edit_text(noneToBlankString(
-                    wireless.GetWirelessProperty(self.networkid, field[0])))
-
-        #FIXME: This causes the entire pile to light up upon use.
-        # Make this into a listbox?
-        self.pile_encrypt = DynWrap(urwid.Pile(theList),attrs=('editbx','editnfc'))
-        self._w.body.body.insert(self._w.body.body.__len__(),self.pile_encrypt)
-        #self._w.body.body.append(self.pile_encrypt)
-
     def ready_widgets(self,ui,body):
         self.ui = ui
         self.body = body
