@@ -3,8 +3,8 @@
 
 """ wicd-curses. (curses/urwid-based) console interface to wicd
 
-Provides the a console UI for wicd, so that people with broken X servers can
-at least get a network connection.  Or those who don't like using X.  ;-)
+Provides a console UI for wicd, so that people with broken X servers can
+at least get a network connection.  Or those who don't like using X and/or GTK.
 
 """
 
@@ -75,7 +75,7 @@ from os import system
 CURSES_REV=wpath.curses_revision
 
 # Fix strings in wicd-curses
-from wicd.translations import language
+from wicd.translations import language, _
 for i in language.keys():
     language[i] = language[i].decode('utf8')
 
@@ -91,13 +91,14 @@ def wrap_exceptions(func):
                 #gobject.source_remove(redraw_tag)
                 loop.quit()
                 ui.stop()
-                print >> sys.stderr, "\n"+language['terminated']
+                print >> sys.stderr, "\n"+_('Terminated by user')
                 #raise
             except DBusException:
-                #gobject.source_remove(redraw_tag)
                 loop.quit()
                 ui.stop()
-                print >> sys.stderr,"\n"+language['dbus_fail']
+                print >> sys.stderr,"\n"+_('DBus failure! '\
+                    'This is most likely caused by the wicd daemon stopping while wicd-curses is running. '\
+                    'Please restart the daemon, and then restart wicd-curses.')
                 raise
             except :
                 # Quit the loop
@@ -106,12 +107,12 @@ def wrap_exceptions(func):
                 # Zap the screen
                 ui.stop()
                 # Print out standard notification:
-                print >> sys.stderr, "\n" + language['exception']
+                # This message was far too scary for humans, so it's gone now.
+                # print >> sys.stderr, "\n" + _('EXCEPTION! Please report this to the maintainer and file a bug report with the backtrace below:')
                 # Flush the buffer so that the notification is always above the
                 # backtrace
                 sys.stdout.flush()
                 # Raise the exception
-                #sleep(2)
                 raise
 
     wrapper.__name__ = func.__name__
@@ -130,7 +131,7 @@ def wrap_exceptions(func):
 def check_for_wired(wired_ip,set_status):
     """ Determine if wired is active, and if yes, set the status. """
     if wired_ip and wired.CheckPluggedIn():
-        set_status(language['connected_to_wired'].replace('$A',wired_ip))
+        set_status(_('Connected to wired network (IP: $A)').replace('$A',wired_ip))
         return True
     else:
         return False
@@ -145,7 +146,7 @@ def check_for_wireless(iwconfig, wireless_ip, set_status):
     if not network:
         return False
 
-    network = unicode(network)
+    network = misc.to_unicode(network)
     if daemon.GetSignalDisplayType() == 0:
         strength = wireless.GetCurrentSignalStrength(iwconfig)
     else:
@@ -153,18 +154,18 @@ def check_for_wireless(iwconfig, wireless_ip, set_status):
 
     if strength is None:
         return False
-    strength = str(strength)            
-    ip = str(wireless_ip)
-    set_status(language['connected_to_wireless'].replace
+    strength = misc.to_unicode(daemon.FormatSignalForPrinting(strength))
+    ip = misc.to_unicode(wireless_ip)
+    set_status(_('Connected to $A at $B (IP: $C)').replace
                     ('$A', network).replace
-                    ('$B', daemon.FormatSignalForPrinting(strength)).replace
-                    ('$C', wireless_ip))
+                    ('$B', strength).replace
+                    ('$C', ip))
     return True
 
 
 # Generate the list of networks.
 # Mostly borrowed/stolen from wpa_cli, since I had no clue what all of those
-# DBUS interfaces do.  ^_^
+# DBUS interfaces do. :P
 # Whatever calls this must be exception-wrapped if it is run if the UI is up
 def gen_network_list():
     wiredL = wired.GetWiredProfileList()
@@ -191,40 +192,41 @@ def about_dialog(body):
 
 ('green',"\\||  \\\\")," |+| ",('green',"//  ||/    \n"),
 ('green'," \\\\\\"),"    |+|    ",('green',"///"),"      http://wicd.net\n",
-('green',"  \\\\\\"),"   |+|   ",('green',"///"),"      ",language["brought_to_you"],"\n",
-('green',"   \\\\\\"),"  |+|  ",('green',"///"),"       Adam Blackburn (wicd)\n",
-"     ___|+|___         Dan O'Reilly   (wicd)\n",
-"    |---------|        Andrew Psaltis (this ui)\n",
+('green',"  \\\\\\"),"   |+|   ",('green',"///"),"      ",_('Brought to you by:'),"\n",
+('green',"   \\\\\\"),"  |+|  ",('green',"///"),"       Adam Blackburn\n",
+"     ___|+|___         Dan O'Reilly\n",
+"    ____|+|____        Andrew Psaltis\n",
+"   |-----------|       David Paleino\n",
 "-----------------------------------------------------"]
-    about = TextDialog(theText,16,55,header=('header','About Wicd'))
+    about = TextDialog(theText,16,55,header=('header',_('About Wicd')))
     about.run(ui,body)
 
 # Modeled after htop's help
 def help_dialog(body):
-    textT  = urwid.Text(('header','wicd-curses help'),'right') 
+    textT  = urwid.Text(('header',_('wicd-curses help')),'right') 
     textSH = urwid.Text(['This is ',('blue','wicd-curses-'+CURSES_REV),' using wicd ',unicode(daemon.Hello()),'\n'])
 
     textH = urwid.Text([
-"For more detailed help, consult the wicd-curses(8) man page.\n",
+_('For more detailed help, consult the wicd-curses(8) man page.')+"\n",
 ('bold','->'),' and ',('bold','<-')," are the right and left arrows respectively.\n"])
 
     text1 = urwid.Text([
-('bold','  H h ?'),": Display this help dialog\n",
-('bold','enter C'),": Connect to selected network\n",
-('bold','      D'),": Disconnect from all networks\n",
-('bold','    ESC'),": Stop a connection in progress\n",
-('bold','   F5 R'),": Refresh network list\n",
-('bold','      P'),": Prefrences dialog\n",
+('bold','  H h ?'),": "+_('Display this help dialog')+"\n",
+('bold','enter C'),": "+_('Connect to selected network')+"\n",
+('bold','      D'),": "+_('Disconnect from all networks')+"\n",
+('bold','    ESC'),": "+_('Stop a connection in progress')+"\n",
+('bold','   F5 R'),": "+_('Refresh network list')+"\n",
+('bold','      P'),": "+_('Preferences dialog')+"\n",
     ])
     text2 = urwid.Text([
-('bold','      I'),": Scan for hidden networks\n",
-('bold','      S'),": Select scripts\n",
-('bold','      O'),": Set up Ad-hoc network\n",
-('bold','     ->'),": Configure selected network\n",
-('bold','      A'),": Display 'about' dialog\n",
-('bold',' F8 q Q'),": Quit wicd-curses\n",
+('bold','      I'),": "+_('Scan for hidden networks')+"\n",
+('bold','      S'),": "+_('Select scripts')+"\n",
+('bold','      O'),": "+_('Set up Ad-hoc network')+"\n",
+('bold','     ->'),": "+_('Configure selected network')+"\n",
+('bold','      A'),": "+_("Display 'about' dialog")+"\n",
+('bold',' F8 q Q'),": "+_('Quit wicd-curses')+"\n",
     ])
-    textF = urwid.Text('Press any key to return.')
+    textF = urwid.Text(_('Press any key to return.'))
     
     # textJ = urwid.Text(('important','Nobody expects the Spanish Inquisition!'))
 
@@ -242,7 +244,11 @@ def help_dialog(body):
         # Don't stop because someone let go of the mouse on the frame
         mouse_release = False
         for k in keys:
-            if urwid.is_mouse_event(k) and k[0] == "mouse release":
+            if urwid.VERSION < (1, 0, 0):
+                check_mouse_event = urwid.is_mouse_event
+            else:
+                check_mouse_event = urwid.util.is_mouse_event
+            if check_mouse_event(k) and k[0] == "mouse release":
                 mouse_release = True
                 break
         if mouse_release :
@@ -254,16 +260,27 @@ def help_dialog(body):
 
 def run_configscript(parent,netname,nettype):
     configfile = wpath.etc+netname+'-settings.conf'
-    header = 'profile' if nettype == 'wired' else 'BSSID'
-    profname = netname if nettype == 'wired' else wireless.GetWirelessProperty(
-            int(netname),'bssid')
-    theText = [ 
-            language['cannot_edit_scripts_1'].replace('$A',configfile).replace('$B',header),
+    if nettype != 'wired':
+        header = 'profile'
+    else:
+        header ='BSSID'
+    if nettype == 'wired':
+        profname = nettype
+    else:
+        profname = wireless.GetWirelessProperty( int(netname),'bssid')
+    theText = [
+    _('To avoid various complications, wicd-curses does not support directly editing the scripts. '\
+      'However, you can edit them manually. First, (as root), open the "$A" config file, and look '\
+      'for the section labeled by the $B in question. In this case, this is:').
+        replace('$A', configfile).replace('$B', header),
 "\n\n["+profname+"]\n\n",
-# Translation needs to be changed to accomidate this text below.
-"""You can also configure the wireless networks by looking for the "[<ESSID>]" field in the config file.  
-
-Once there, you can adjust (or add) the "beforescript", "afterscript", "predisconnectscript" and "postdisconnectscript" variables as needed, to change the preconnect, postconnect, predisconnect and postdisconnect scripts respectively.  Note that you will be specifying the full path to the scripts - not the actual script contents.  You will need to add/edit the script contents separately.  Refer to the wicd manual page for more information."""]
+    _('You can also configure the wireless networks by looking for the "[<ESSID>]" field in the config file.'),
+    _('Once there, you can adjust (or add) the "beforescript", "afterscript", "predisconnectscript" '\
+      'and "postdisconnectscript" variables as needed, to change the preconnect, postconnect, '\
+      'predisconnect and postdisconnect scripts respectively.  Note that you will be specifying '\
+      'the full path to the scripts - not the actual script contents.  You will need to add/edit '\
+      'the script contents separately.  Refer to the wicd manual page for more information.')
+    ]
     dialog = TextDialog(theText,20,80)
     dialog.run(ui,parent)
     # This code works with many distributions, but not all of them.  So, to
@@ -329,7 +346,12 @@ class NetLabel(urwid.WidgetWrap):
                 str(wireless.GetWirelessProperty(id, strenstr)))
         self.essid = wireless.GetWirelessProperty(id, 'essid')
         self.bssid = wireless.GetWirelessProperty(id, 'bssid')
-        self.encrypt = wireless.GetWirelessProperty(id,'encryption_method') if wireless.GetWirelessProperty(id, 'encryption') else language['unsecured']
+
+        if wireless.GetWirelessProperty(id, 'encryption'):
+            self.encrypt = wireless.GetWirelessProperty(id,'encryption_method')
+        else:
+            self.encrypt = _('Unsecured')
+
         self.mode  = wireless.GetWirelessProperty(id, 'mode') # Master, Ad-Hoc
         self.channel = wireless.GetWirelessProperty(id, 'channel')
         theString = '  %-*s %25s %9s %17s %6s %4s' % (gap,
@@ -353,7 +375,7 @@ class WiredComboBox(ComboBox):
     list : the list of wired network profiles.  The rest is self-explanitory.
     """
     def __init__(self,list):
-        self.ADD_PROFILE = '---'+language["add_new_profile"]+'---'
+        self.ADD_PROFILE = '---'+_('Add a new profile')+'---'
         self.__super.__init__(use_enter=False)
         self.set_list(list)
 
@@ -383,21 +405,27 @@ class WiredComboBox(ComboBox):
 
     def keypress(self,size,key):
         prev_focus = self.get_focus()[1]
-        key = self.__super.keypress(size,key)
-        if self.get_focus()[1] == len(self.list)-1:
-            dialog = InputDialog(('header',language["add_new_wired_profile"]),7,30)
-            
-            exitcode,name = dialog.run(ui,self.parent)
-            if exitcode == 0:
-                wired.CreateWiredNetworkProfile(name,False)
-                self.set_list(wired.GetWiredProfileList())
-                self.rebuild_combobox()
-            self.set_focus(prev_focus)
-        else:
-            wired.ReadWiredNetworkProfile(self.get_selected_profile())
+        key = ComboBox.keypress(self,size,key)
+        if key == ' ':
+            if self.get_focus()[1] == len(self.list)-1:
+                dialog = InputDialog(('header',_('Add a new wired profile')),7,30)
+                exitcode,name = dialog.run(ui,self.parent)
+                if exitcode == 0:
+                    name = name.strip()
+                    if not name:
+                        error(ui,self.parent,'Invalid profile name')
+                        self.set_focus(prev_focus)
+                        return key
+
+                    wired.CreateWiredNetworkProfile(name,False)
+                    self.set_list(wired.GetWiredProfileList())
+                    self.rebuild_combobox()
+                self.set_focus(prev_focus)
+            else:
+                wired.ReadWiredNetworkProfile(self.get_selected_profile())
         if key == 'delete':
             if len(self.theList) == 1:
-                error(self.ui,self.parent,language["no_delete_last_profile"])
+                error(self.ui,self.parent,_('wicd-curses does not support deleting the last wired profile.  Try renaming it ("F2")'))
                 return key
             wired.DeleteWiredNetworkProfile(self.get_selected_profile())
             # Return to the top of the list if something is deleted.
@@ -410,7 +438,7 @@ class WiredComboBox(ComboBox):
             self.set_list(wired.GetWiredProfileList())
             self.rebuild_combobox()
         if key == 'f2':
-            dialog = InputDialog(('header',language["rename_wired_profile"]),7,30,
+            dialog = InputDialog(('header',_('Rename wired profile')),7,30,
                     edit_text=unicode(self.get_selected_profile()))
             exitcode,name = dialog.run(ui,self.parent)
             if exitcode == 0:
@@ -430,12 +458,12 @@ class WiredComboBox(ComboBox):
 # Dialog2 that initiates an Ad-Hoc network connection
 class AdHocDialog(Dialog2):
     def __init__(self):
-        essid_t = language['essid']
-        ip_t = language['ip']
-        channel_t = language['channel']
-        key_t = "    " + language['key']
-        use_ics_t = language['use_ics']
-        use_encrypt_t = language['use_wep_encryption']
+        essid_t = _('ESSID')
+        ip_t = _('IP')
+        channel_t = _('Channel')
+        key_t = "    " + _('Key')
+        use_ics_t =  _('Activate Internet Connection Sharing')
+        use_encrypt_t = _('Use Encryption (WEP only)')
 
         self.essid_edit = DynEdit(essid_t)
         self.ip_edit = DynEdit(ip_t)
@@ -455,14 +483,11 @@ class AdHocDialog(Dialog2):
 
         l = [self.essid_edit,self.ip_edit,self.channel_edit,blank,
                 self.use_ics_chkb,self.use_encrypt_chkb,self.key_edit]
-        #for line in text:
-        #    l.append( urwid.Text( line,align=align))
         body = urwid.ListBox(l)
-        #body = urwid.AttrWrap(body, 'body')
 
-        header = ('header',language['create_adhoc_network'])
+        header = ('header', _('Create an Ad-Hoc Network'))
         Dialog2.__init__(self, header, 15, 50, body)
-        self.add_buttons([('OK',1),('Cancel',-1)])
+        self.add_buttons([(_('OK'),1),(_('Cancel'),-1)])
         self.frame.set_focus('body')
 
     def encrypt_callback(self,chkbox,new_state,user_info=None):
@@ -480,12 +505,11 @@ class AdHocDialog(Dialog2):
             self.view.keypress( size, k )
     def on_exit(self,exitcode):
         data = ( self.essid_edit.get_edit_text(),
-                 self.ip_edit.get_edit_text(),
+                 self.ip_edit.get_edit_text().strip(),
                  self.channel_edit.get_edit_text(),
                  self.use_ics_chkb.get_state(),
                  self.use_encrypt_chkb.get_state(),
                  self.key_edit.get_edit_text())
-
         return exitcode, data
 
 ########################################
@@ -499,22 +523,25 @@ class appGUI():
         self.size = ui.get_cols_rows()
         # Happy screen saying that you can't do anything because we're scanning
         # for networks.  :-)
-        self.screen_locker = urwid.Filler(urwid.Text(('important',language['scanning_stand_by']), align='center'))
-        self.no_wlan = urwid.Filler(urwid.Text(('important',language['no_wireless_networks_found']), align='center'))
-        self.TITLE = language['wicd_curses']
+        self.screen_locker = urwid.Filler(urwid.Text(('important',_('Scanning networks... stand by...')), align='center'))
+        self.no_wlan = urwid.Filler(urwid.Text(('important',_('No wireless networks found.')), align='center'))
+        self.TITLE = _('Wicd Curses Interface')
         self.WIRED_IDX = 1
         self.WLESS_IDX = 3
 
         header = urwid.AttrWrap(urwid.Text(self.TITLE,align='right'), 'header')
-        self.wiredH=urwid.Filler(urwid.Text("Wired Network(s)"))
+        self.wiredH=urwid.Filler(urwid.Text(_('Wired Networks')))
         self.list_header=urwid.AttrWrap(urwid.Text(gen_list_header()),'listbar')
-        self.wlessH=NSelListBox([urwid.Text("Wireless Network(s)"),self.list_header])
+        self.wlessH=NSelListBox([urwid.Text(_('Wireless Networks')),self.list_header])
+
+        # Init this earlier to make update_status happy
+        self.update_tag = None
 
         # FIXME: This should be two variables
         self.focusloc = [1,0]
 
         # These are empty to make sure that things go my way.
-        wiredL,wlessL = [],[]# = gen_network_list()
+        wiredL,wlessL = [],[]
 
         self.frame = None
         self.diag = None
@@ -525,23 +552,22 @@ class appGUI():
         
         # Keymappings proposed by nanotube in #wicd
         keys = [
-                ('H' ,'Help'  ,None),
-                ('right','Config',None),
+                ('H' ,_('Help'),None),
+                ('right',_('Config'),None),
                 #('  ','         ',None),
-                ('C' ,'Connect',None),
-                ('D' ,'Disconn',None),
-                ('R' ,'Refresh',None),
-                ('P' ,'Prefs',None),
-                ('I' ,'Hidden',None),
-                ('A' ,'About',None),
-                ('Q' ,'Quit',loop.quit)
+                ('K' , _('RfKill'),None),
+                ('C' ,_('Connect'),None),
+                ('D' ,_('Disconn'),None),
+                ('R' ,_('Refresh'),None),
+                ('P' ,_('Prefs'),None),
+                ('I' ,_('Hidden'),None),
+                ('A' ,_('About'),None),
+                ('Q' ,_('Quit'),loop.quit)
                ]
 
         self.primaryCols = OptCols(keys,self.handle_keys)
-        self.time_label = \
-                  urwid.AttrWrap(urwid.Text(strftime('%H:%M:%S')), 'timebar')
-        self.status_label = urwid.AttrWrap(urwid.Text('blah'),'important')
-        self.footer2 = urwid.Columns([self.status_label,('fixed', 8, self.time_label)])
+        self.status_label = urwid.AttrWrap(urwid.Text(''),'important')
+        self.footer2 = urwid.Columns([self.status_label])
         self.footerList = urwid.Pile([self.primaryCols,self.footer2])
 
         self.frame = urwid.Frame(self.thePile,
@@ -565,18 +591,20 @@ class appGUI():
 
         self.update_status()
 
+        #self.max_wait = ui.max_wait
+
     def doScan(self, sync=False):
         self.scanning = True
         wireless.Scan(False)
 
     def init_other_optcols(self):
         # The "tabbed" preferences dialog
-        self.prefCols = OptCols( [ ('meta enter','OK'),
-                                   ('meta [','Tab Left',),
-                                   ('meta ]','Tab Right'),
-                                   ('esc','Cancel') ], self.handle_keys)
-        self.confCols = OptCols( [ ('meta enter','OK'),
-                                   ('esc','Cancel') ],self.handle_keys)
+        self.prefCols = OptCols( [ ('f10',_('OK')),
+                                   ('page up',_('Tab Left'),),
+                                   ('page down', _('Tab Right')),
+                                   ('esc',_('Cancel')) ], self.handle_keys)
+        self.confCols = OptCols( [ ('f10',_('OK')),
+                                   ('esc',_('Cancel')) ],self.handle_keys)
 
     # Does what it says it does
     def lock_screen(self):
@@ -598,7 +626,7 @@ class appGUI():
         self.update_ui()
 
     def raise_hidden_network_dialog(self):
-        dialog = InputDialog(('header',language["select_hidden_essid"]),7,30,language['scan'])
+        dialog = InputDialog(('header',_('Select Hidden Network ESSID')),7,30,_('Scan'))
         exitcode,hidden = dialog.run(ui,self.frame)
         if exitcode != -1:
             # That dialog will sit there for a while if I don't get rid of it
@@ -694,7 +722,7 @@ class appGUI():
         if self.connecting: 
             if not self.conn_status:
                 self.conn_status = True
-                gobject.idle_add(self.set_connecting_status,fast)
+                gobject.timeout_add(250,self.set_connecting_status,fast)
             return True
         else:
             if check_for_wired(wired.GetWiredIP(''),self.set_status):
@@ -707,7 +735,7 @@ class appGUI():
                     self.set_status):
                 return True
             else:
-                self.set_status(language['not_connected'])
+                self.set_status(_('Not connected'))
                 self.update_ui()
                 return True
 
@@ -719,17 +747,12 @@ class appGUI():
                 iwconfig = wireless.GetIwconfig()
             else:
                 iwconfig = ''
-            # set_status is rigged to return false when it is not
-            # connecting to anything, so this should work.
-            return self.set_status(wireless.GetCurrentNetwork(iwconfig) +
-                    ': ' +
-                    language[str(wireless.CheckWirelessConnectingMessage())],
-                    True)
+            essid = wireless.GetCurrentNetwork(iwconfig)
+            stat = wireless.CheckWirelessConnectingMessage()
+            return self.set_status("%s: %s" % (essid, stat), True)
         if wired_connecting:
-            return self.set_status( language['wired_network'] +
-                    ': ' +
-                    language[str(wired.CheckWiredConnectingMessage())],
-                    True)
+            return self.set_status(_('Wired Network') +
+                    ': ' + wired.CheckWiredConnectingMessage(), True)
         else:
             self.conn_status=False
             return False
@@ -756,18 +779,9 @@ class appGUI():
             self.tcount+=1
             toAppend=self.twirl[self.tcount % 4]
         self.status_label.set_text(text+' '+toAppend)
+        self.update_ui()
         return True
 
-    # Make sure the screen is still working by providing a pretty counter.
-    # Not necessary in the end, but I will be using footer1 for stuff in
-    # the long run, so I might as well put something there.
-    #@wrap_exceptions
-    def update_time(self):
-        self.time_label.set_text(strftime('%H:%M:%S'))
-        return True
-
-    # Yeah, I'm copying code.  Anything wrong with that?
-    #@wrap_exceptions
     def dbus_scan_finished(self):
         # I'm pretty sure that I'll need this later.
         #if not self.connecting:
@@ -775,8 +789,6 @@ class appGUI():
         self.unlock_screen()
         self.scanning = False
 
-    # Same, same, same, same, same, same
-    #@wrap_exceptions
     def dbus_scan_started(self):
         self.scanning = True
         if self.diag_type == 'conf':
@@ -803,6 +815,9 @@ class appGUI():
             if "f5" in keys or 'R' in keys:
                 self.lock_screen()
                 self.doScan()
+            if 'k' in keys or 'K' in keys:
+                wireless.SwitchRfKill()
+                self.update_netlist()
             if "D" in keys:
                 # Disconnect from all networks.
                 daemon.Disconnect()
@@ -812,7 +827,7 @@ class appGUI():
                     focus = self.thePile.get_focus()
                     self.frame.set_footer(urwid.Pile([self.confCols,self.footer2]))
                     if focus == self.wiredCB:
-                        self.diag = WiredSettingsDialog(self.wiredCB.get_body().get_selected_profile())
+                        self.diag = WiredSettingsDialog(self.wiredCB.get_body().get_selected_profile(),self.frame)
                         self.frame.set_body(self.diag)
                     else:
                         # wireless list only other option
@@ -879,7 +894,11 @@ class appGUI():
                                                 data[4], False)
             
         for k in keys:
-            if urwid.is_mouse_event(k):
+            if urwid.VERSION < (1, 0, 0):
+                check_mouse_event = urwid.is_mouse_event
+            else:
+                check_mouse_event = urwid.util.is_mouse_event
+            if check_mouse_event(k):
                 event, button, col, row = k
                 self.frame.mouse_event( self.size,
                         event, button, col, row,
@@ -890,7 +909,7 @@ class appGUI():
                 if  k == 'esc' or k == 'q' or k == 'Q':
                     self.restore_primary()
                     break
-                if k == 'meta enter':
+                if k == 'f10':
                     self.diag.save_settings()
                     self.restore_primary()
                     break
@@ -898,34 +917,32 @@ class appGUI():
                 self.size = ui.get_cols_rows()
                 continue
 
+    def call_update_ui(self,source,cb_condition):           
+        self.update_ui(True)                                
+        return True                                         
+                
     # Redraw the screen
     @wrap_exceptions
-    def update_ui(self):
-        #self.update_status()
-        canvas = self.frame.render( (self.size),True )
-        ###  GRRRRRRRRRRRRRRRRRRRRR           ->^^^^
-        # It looks like if I want to get the statusbar to update itself
-        # continuously, I would have to use overlay the canvasses and redirect
-        # the input.  I'll try to get that working at a later time, if people
-        # want that "feature".
-        #canvaso = urwid.CanvasOverlay(self.dialog.render( (80,20),True),canvas,0,1)
-        # If the screen is turned off for some reason, don't even try to do the
-        # rest of the stuff.
+    def update_ui(self,from_key=False):
         if not ui._started:
             return False
+
+        input_data = ui.get_input_nonblocking()
+        # Resolve any "alarms" in the waiting
+        self.handle_keys(input_data[1])
+
+        # Update the screen
+        canvas = self.frame.render( (self.size),True )
         ui.draw_screen((self.size),canvas)
-        keys = ui.get_input()
-        self.handle_keys(keys)
-            
-        return True
+        # Get the input data
+        if self.update_tag != None:
+            gobject.source_remove(self.update_tag)
+        #if from_key:
+        return False
 
     def connect(self, nettype, networkid, networkentry=None):
         """ Initiates the connection process in the daemon. """
         if nettype == "wireless":
-            #if not self.check_encryption_valid(networkid,
-            #                                   networkentry.advanced_dialog):
-            #    self.edit_advanced(None, None, nettype, networkid, networkentry)
-            #    return False
             wireless.ConnectWireless(networkid)
         elif nettype == "wired":
             wired.ConnectWired()
@@ -937,18 +954,11 @@ class appGUI():
 
 def main():
     global ui, dlogger
-    # We are _not_ python.
+    # We are not python.
     misc.RenameProcess('wicd-curses')
 
-    # Import the screen based on whatever the user picked.
-    # The raw_display will have some features that may be useful to users
-    # later
-    if options.screen == 'raw':
-        import urwid.raw_display
-        ui = urwid.raw_display.Screen()
-    elif options.screen == 'curses':
-        import urwid.curses_display
-        ui = urwid.curses_display.Screen()
+    import urwid.raw_display
+    ui = urwid.raw_display.Screen()
 
     #if options.debug:
     #    dlogger = logging.getLogger("Debug")
@@ -973,7 +983,6 @@ def main():
         ('editnfc','brown','default','bold'),
         ('tab active','dark green','light gray'),
         ('infobar','light gray','dark blue'),
-        ('timebar','dark gray','default'),
         ('listbar','light blue','default'),
         # Simple colors around text
         ('green','dark green','default'),
@@ -1001,32 +1010,32 @@ def run():
     # I've left this commented out many times.
     bus.add_signal_receiver(app.update_netlist, 'StatusChanged',
                             'org.wicd.daemon')
-    # Update what the interface looks like as an idle function
-    gobject.idle_add(app.update_ui)
-    # Update the connection status on the bottom every 1.5 s.
+    # Update the connection status on the bottom every 2 s.
     gobject.timeout_add(2000,app.update_status)
-    # This will make sure that it is updated on the second.
-    gobject.timeout_add(500,app.update_time)
+
+    # Get input file descriptors and add callbacks to the ui-updating function
+    fds = ui.get_input_descriptors()
+    for fd in fds:
+        gobject.io_add_watch(fd, gobject.IO_IN,app.call_update_ui)
+    app.update_ui()
     loop.run()
 
 # Mostly borrowed from gui.py
 def setup_dbus(force=True):
-    global bus, daemon, wireless, wired, DBUS_AVAIL
+    global bus, daemon, wireless, wired
     try:
         dbusmanager.connect_to_dbus()
     except DBusException:
-        # I may need to be a little more verbose here.
-        # Suggestions as to what should go here, please?
-        print >> sys.stderr, language['cannot_connect_to_daemon']
-        #raise
-        # return False # <- Will need soon.
+        print >> sys.stderr, _("Can't connect to the daemon, trying to start it automatically...")
     bus = dbusmanager.get_bus()
     dbus_ifaces = dbusmanager.get_dbus_ifaces()
     daemon = dbus_ifaces['daemon']
     wireless = dbus_ifaces['wireless']
     wired = dbus_ifaces['wired']
-    DBUS_AVAIL = True
-    
+
+    if not daemon:
+        print 'Error connecting to wicd via D-Bus.  Please make sure the wicd service is running.'
+        sys.exit(3)
 
     netentry_curses.dbus_init(dbus_ifaces)
     return True
@@ -1038,21 +1047,17 @@ setup_dbus()
 ########################################
 if __name__ == '__main__':
     try:
-        parser = OptionParser(version="wicd-curses-%s (using wicd %s)" % (CURSES_REV,daemon.Hello()))
+        parser = OptionParser(version="wicd-curses-%s (using wicd %s)" % (CURSES_REV,daemon.Hello()), prog="wicd-curses")
     except Exception, e:
         if "DBus.Error.AccessDenied" in e.get_dbus_name():
-            print language['access_denied_wc'].replace('$A','\033[1;34m'+wpath.wicd_group+'\033[0m')
+            print _('ERROR: wicd-curses was denied access to the wicd daemon: '\
+                'please check that your user is in the "$A" group.').\
+                replace('$A','\033[1;34m' + wpath.wicd_group + '\033[0m')
             sys.exit(1)
         else:
             raise
-    parser.set_defaults(screen='raw',debug=False)
-    parser.add_option("-r", "--raw-screen",action="store_const",const='raw'
-            ,dest='screen',help="use urwid's raw screen controller (default)")
-    parser.add_option("-c", "--curses-screen",action="store_const",const='curses',dest='screen',help="use urwid's curses screen controller")
-    parser.add_option("-d", "--debug",action="store_true"
-            ,dest='debug',help="enable logging of wicd-curses (currently does nothing)")
+    #parser.add_option("-d", "--debug",action="store_true"
+    #        ,dest='debug',help="enable logging of wicd-curses (currently does nothing)")
+
     (options,args) = parser.parse_args()
     main()
-    # Make sure that the terminal does not try to overwrite the last line of
-    # the program, so that everything looks pretty.
-    #print ""
