@@ -31,6 +31,9 @@ from threading import Thread
 from subprocess import Popen, STDOUT, PIPE, call
 from commands import getoutput
 from itertools import repeat, chain, izip
+from pipes import quote
+
+from wicd.translations import _
 
 # wicd imports
 import wpath
@@ -70,6 +73,30 @@ _sudo_dict = {
     KTSUSS: "ktsuss",
 }
 
+_status_dict = {
+    'aborted': _('Connection Cancelled'),
+    'association_failed': _('Connection failed: Could not contact the wireless access point.'),
+    'bad_pass': _('Connection Failed: Bad password'),
+    'configuring_interface': _('Configuring wireless interface...'),
+    'dhcp_failed': _('Connection Failed: Unable to Get IP Address'),
+    'done': _('Done connecting...'),
+    'failed': _('Connection Failed.'),
+    'flushing_routing_table': _('Flushing the routing table...'),
+    'generating_psk': _('Generating PSK...'),
+    'generating_wpa_config': _('Generating WPA configuration file...'),
+    'interface_down': _('Putting interface down...'),
+    'interface_up': _('Putting interface up...'),
+    'no_dhcp_offers': _('Connection Failed: No DHCP offers received.'),
+    'resetting_ip_address': _('Resetting IP address...'),
+    'running_dhcp': _('Obtaining IP address...'),
+    'setting_broadcast_address': _('Setting broadcast address...'),
+    'setting_static_dns': _('Setting static DNS servers...'),
+    'setting_static_ip': _('Setting static IP addresses...'),
+    'success': _('Connection successful.'),
+    'validating_authentication': _('Validating authentication...'),
+    'verifying_association': _('Verifying access point association...'),
+}
+
 class WicdError(Exception):
     pass
     
@@ -143,14 +170,17 @@ def LaunchAndWait(cmd):
 
 def IsValidIP(ip):
     """ Make sure an entered IP is valid. """
-    if ip != None:
-        if ip.count('.') == 3:
-            ipNumbers = ip.split('.')
-            for number in ipNumbers:
-                if not number.isdigit() or int(number) > 255:
-                    return False
-            return ipNumbers
-    return False
+    if not ip: return False
+
+    ipNumbers = ip.split('.')
+    if len(ipNumbers) < 4:
+        return False
+
+    for number in ipNumbers:
+        if not number.isdigit() or int(number) > 255:
+            return False
+
+    return ipNumbers
 
 def PromptToStartDaemon():
     """ Prompt the user to start the daemon """
@@ -163,7 +193,7 @@ def PromptToStartDaemon():
     else:
         msg = '--caption'
     sudo_args = [sudo_prog, msg, 
-                 'Wicd needs to access your computer\'s network cards.',
+                 _("Wicd needs to access your computer's network cards."),
                  daemonloc]
     os.spawnvpe(os.P_WAIT, sudo_prog, sudo_args, os.environ)
     return True
@@ -180,20 +210,27 @@ def WriteLine(my_file, text):
     """ write a line to a file """
     my_file.write(text + "\n")
 
-def ExecuteScripts(scripts_dir, verbose=False):
+def ExecuteScripts(scripts_dir, verbose=False, extra_parameters=()):
     """ Execute every executable file in a given directory. """
     if not os.path.exists(scripts_dir):
         return
-    for obj in os.listdir(scripts_dir):
+    for obj in sorted(os.listdir(scripts_dir)):
+        if obj.startswith(".") or obj.endswith(("~", ".new", ".orig")):
+            continue
         obj = os.path.abspath(os.path.join(scripts_dir, obj))
         if os.path.isfile(obj) and os.access(obj, os.X_OK):
-            ExecuteScript(os.path.abspath(obj), verbose=verbose)
+            ExecuteScript(os.path.abspath(obj), verbose=verbose,
+                          extra_parameters=extra_parameters)
 
-def ExecuteScript(script, verbose=False):
+def ExecuteScript(script, verbose=False, extra_parameters=()):
     """ Execute a command and send its output to the bit bucket. """
+    extra_parameters = [ quote(s) for s in extra_parameters ]
+    params = ' '.join(extra_parameters)
+    # escape script name
+    script = quote(script)
     if verbose:
-        print "Executing %s" % script
-    ret = call("%s > /dev/null 2>&1" % script, shell=True)
+        print "Executing %s with params %s" % (script, params)
+    ret = call('%s %s > /dev/null 2>&1' % (script, params), shell=True)
     if verbose:
         print "%s returned %s" % (script, ret)
 
@@ -234,7 +271,10 @@ def ParseEncryption(network):
     """
     enctemplate = open(wpath.encryption + network["enctype"])
     template = enctemplate.readlines()
-    config_file = "ap_scan=1\n"
+    if network.get('essid'):
+        config_file = "ap_scan=1\n"
+    else:
+        config_file = "ap_scan=0\n"
     should_replace = False
     for index, line in enumerate(template):
         if not should_replace:
@@ -249,7 +289,7 @@ def ParseEncryption(network):
                 if cur_val:
                     if cur_val[0] == 'SCAN':
                         #TODO should this be hardcoded?
-                        line = line.replace("$_SCAN", "0")
+                        line = line.replace("$_SCAN", "1")
                         config_file = ''.join([config_file, line])
                     else:
                         rep_val = network.get(cur_val[0].lower())
@@ -266,8 +306,11 @@ def ParseEncryption(network):
 
     # Write the data to the files then chmod them so they can't be read 
     # by normal users.
-    file_loc = os.path.join(wpath.networks,
-                            network['bssid'].replace(":", "").lower())
+    if network.get('bssid'):
+        file_name = network['bssid'].replace(":", "").lower()
+    else:
+        file_name = 'wired'
+    file_loc = os.path.join(wpath.networks, file_name)
     f = open(file_loc, "w")
     os.chmod(file_loc, 0600)
     os.chown(file_loc, 0, 0)
@@ -276,7 +319,7 @@ def ParseEncryption(network):
     f.write(config_file)
     f.close()
 
-def LoadEncryptionMethods():
+def LoadEncryptionMethods(wired = False):
     """ Load encryption methods from configuration files
 
     Loads all the encryption methods from the template files
@@ -284,8 +327,12 @@ def LoadEncryptionMethods():
     loaded, the template must be listed in the "active" file.
 
     """
+    if wired:
+        active_fname = "active_wired"
+    else:
+        active_fname = "active"
     try:
-        enctypes = open(wpath.encryption + "active","r").readlines()
+        enctypes = open(wpath.encryption + active_fname,"r").readlines()
     except IOError, e:
         print "Fatal Error: template index file is missing."
         raise IOError(e)
@@ -327,6 +374,7 @@ def _parse_enc_template(enctype):
     cur_type["fields"] = []
     cur_type['optional'] = []
     cur_type['required'] = []
+    cur_type['protected'] = []
     cur_type['name'] = ""
     for index, line in enumerate(f):
         if line.startswith("name") and not cur_type["name"]:
@@ -345,6 +393,13 @@ def _parse_enc_template(enctype):
                 # An error occured parsing the optional line.
                 print "Invalid 'optional' line found in template %s" % enctype
                 continue
+        elif line.startswith("protected"):
+            cur_type["protected"] = __parse_field_ent(parse_ent(line, "protected"),
+                                                    field_type="protected")
+            if not cur_type["protected"]:
+                # An error occured parsing the protected line.
+                print "Invalid 'protected' line found in template %s" % enctype
+                continue
         elif line.startswith("----"):
             # We're done.
             break
@@ -368,7 +423,19 @@ def noneToString(text):
     if text in (None, ""):
         return "None"
     else:
-        return str(text)
+        return to_unicode(text)
+
+def sanitize_escaped(s):
+    """ Sanitize double-escaped unicode strings. """
+    lastpos = 0
+    while True:
+        lastpos = s.find('\\x', lastpos + 1)
+        #print lastpos
+        if lastpos == -1:
+            break
+        c = s[lastpos+2:lastpos+4]  # i.e. get the next two characters
+        s = s.replace('\\x'+c, chr(int(c, 16)))
+    return s
 
 def to_unicode(x):
     """ Attempts to convert a string to utf-8. """
@@ -377,6 +444,9 @@ def to_unicode(x):
         return x
     if isinstance(x, unicode):
         return x.encode('utf-8')
+
+    x = sanitize_escaped(x)
+
     encoding = locale.getpreferredencoding()
     try:
         ret = x.decode(encoding).encode('utf-8')
@@ -393,16 +463,13 @@ def to_unicode(x):
     
 def RenameProcess(new_name):
     """ Renames the process calling the function to the given name. """
-    if sys.platform != 'linux2':
+    if 'linux' not in sys.platform:
         print 'Unsupported platform'
         return False
     try:
         import ctypes
-        is_64 = os.path.exists('/lib64/libc.so.6')
-        if is_64:
-            libc = ctypes.CDLL('/lib64/libc.so.6')
-        else:
-            libc = ctypes.CDLL('/lib/libc.so.6')
+        from ctypes.util import find_library
+        libc = ctypes.CDLL(find_library('c'))
         libc.prctl(15, new_name, 0, 0, 0)
         return True
     except:
