@@ -16,6 +16,7 @@ class BaseWirelessInterface() -- Control a wireless network interface.
 #   Copyright (C) 2007 - 2009 Adam Blackburn
 #   Copyright (C) 2007 - 2009 Dan O'Reilly
 #   Copyright (C) 2007 - 2009 Byron Hillis
+#   Copyright (C) 2009        Andrew Psaltis
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License Version 2 as
@@ -44,12 +45,12 @@ from misc import find_path
 _re_mode = (re.I | re.M | re.S)
 essid_pattern = re.compile('.*ESSID:"?(.*?)"?\s*\n', _re_mode)
 ap_mac_pattern = re.compile('.*Address: (.*?)\n', _re_mode)
-channel_pattern = re.compile('.*Channel:?=? ?(\d\d?)', _re_mode)
+channel_pattern = re.compile('.*Channel:?=? ?(\d+)', _re_mode)
 strength_pattern = re.compile('.*Quality:?=? ?(\d+)\s*/?\s*(\d*)', _re_mode)
 altstrength_pattern = re.compile('.*Signal level:?=? ?(\d+)\s*/?\s*(\d*)', _re_mode)
 signaldbm_pattern = re.compile('.*Signal level:?=? ?(-\d\d*)', _re_mode)
 bitrates_pattern = re.compile('(\d+\s+\S+/s)', _re_mode)
-mode_pattern = re.compile('.*Mode:(.*?)\n', _re_mode)
+mode_pattern = re.compile('.*Mode:([A-Za-z-]*?)\n', _re_mode)
 freq_pattern = re.compile('.*Frequency:(.*?)\n', _re_mode)
 wep_pattern = re.compile('.*Encryption key:(.*?)\n', _re_mode)
 altwpa_pattern = re.compile('(wpa_ie)', _re_mode)
@@ -57,9 +58,11 @@ wpa1_pattern = re.compile('(WPA Version 1)', _re_mode)
 wpa2_pattern = re.compile('(WPA2)', _re_mode)
 
 #iwconfig-only regular expressions.
+ip_up = re.compile(r'flags=[0.9]*<([^>]*)>', re.S)
 ip_pattern = re.compile(r'inet [Aa]d?dr[^.]*:([^.]*\.[^.]*\.[^.]*\.[0-9]*)', re.S)
-bssid_pattern = re.compile('.*Access Point: (([0-9A-Z]{2}:){5}[0-9A-Z]{2})', _re_mode)
-bitrate_pattern = re.compile('.*Bit Rate=(.*?/s)', _re_mode)
+ip_pattern1 = re.compile(r'inet ([^.]*\.[^.]*\.[^.]*\.[0-9]*)', re.S)
+bssid_pattern = re.compile('.*[(Access Point)|(Cell)]: (([0-9A-Z]{2}:){5}[0-9A-Z]{2})', _re_mode)
+bitrate_pattern = re.compile('.*Bit Rate[=:](.*?/s)', _re_mode)
 opmode_pattern = re.compile('.*Mode:(.*?) ', _re_mode)
 authmethods_pattern = re.compile('.*Authentication capabilities :\n(.*?)Current', _re_mode)
 
@@ -155,6 +158,20 @@ def NeedsExternalCalls():
     """ Returns True if the backend needs to use an external program. """
     raise NotImplementedError
 
+def GetWpaSupplicantDrivers():
+    """ Returns a list of all valid wpa_supplicant drivers. """
+    output = misc.Run(["wpa_supplicant", "-h"])
+    try:
+        output = output.split("drivers:")[1].split("options:")[0].strip()
+    except:
+        print "Warning: Couldn't get list of valid wpa_supplicant drivers"
+        return [""]
+    patt = re.compile("(\S+)\s+=.*")
+    drivers = patt.findall(output) or [""]
+    # We cannot use the "wired" driver for wireless interfaces.
+    if 'wired' in drivers:
+        drivers.remove('wired')
+    return drivers
 def IsValidWpaSuppDriver(driver):
     """ Returns True if given string is a valid wpa_supplicant driver. """
     output = misc.Run(["wpa_supplicant", "-D%s" % driver, "-iolan19",
@@ -230,7 +247,7 @@ class BaseInterface(object):
         return path
 
     
-    def _get_dhcp_command(self, flavor=None):
+    def _get_dhcp_command(self, flavor=None, hostname=None):
         """ Returns the correct DHCP client command. 
        
         Given a type of DHCP request (create or release a lease),
@@ -258,36 +275,72 @@ class BaseInterface(object):
                 client = None
                 cmd = ""
             return (client, cmd)
+
+                # probably /var/lib/wicd/dhclient.conf with defaults
+        dhclient_conf_path = os.path.join(
+                    wpath.varlib,
+                    'dhclient.conf'
+                )
         
         client_dict = {
             "dhclient" : 
-                {'connect' : r"%(cmd)s %(iface)s",
+                {'connect' : r"%(cmd)s -cf %(dhclientconf)s %(iface)s",
                  'release' : r"%(cmd)s -r %(iface)s",
                  'id' : misc.DHCLIENT, 
                  },
             "pump" : 
-                { 'connect' : r"%(cmd)s -i %(iface)s",
+                { 'connect' : r"%(cmd)s -i %(iface)s -h %(hostname)s",
                   'release' : r"%(cmd)s -r -i %(iface)s",
                   'id' : misc.PUMP,
                 },
             "dhcpcd" : 
-                {'connect' : r"%(cmd)s %(iface)s",
+                {'connect' : r"%(cmd)s -h %(hostname)s --noipv4ll %(iface)s ",
                  'release' : r"%(cmd)s -k %(iface)s",
                  'id' : misc.DHCPCD,
                 },
             "udhcpc":
-                {'connect' : r"%(cmd)s -n -i %(iface)s",
+                {'connect' : r"%(cmd)s -n -i %(iface)s -H %(hostname)s ",
                  'release' : r"killall -SIGUSR2 %(cmd)s",
                  'id' : misc.UDHCPC,
                 },
         }
         (client_name, cmd) = get_client_name(self.DHCP_CLIENT)
+
+        # cause dhclient doesn't have a handy dandy argument
+        # for specifing the hostname to be sent
+        if client_name == "dhclient" and flavor:
+            if hostname == None:
+                # <hostname> will use the system hostname
+                # we'll use that if there is hostname passed
+                # that shouldn't happen, though
+                hostname = '<hostname>'
+            print 'attempting to set hostname with dhclient'
+            print 'using dhcpcd or another supported client may work better'
+            dhclient_template = \
+                open(os.path.join(wpath.etc, 'dhclient.conf.template'), 'r')
+
+            output_conf = open(dhclient_conf_path, 'w')
+
+            for line in dhclient_template.readlines():
+                line = line.replace('$_HOSTNAME', hostname)
+                output_conf.write(line)
+
+            output_conf.close()
+            dhclient_template.close()
+            os.chmod(dhclient_conf_path, 0644)
+
         if not client_name or not cmd:
             print "WARNING: Failed to find a valid dhcp client!"
             return ""
             
         if flavor == "connect":
-            return client_dict[client_name]['connect'] % {"cmd":cmd, "iface":self.iface}
+            if not hostname:
+                hostname = os.uname()[1]
+            return client_dict[client_name]['connect'] % \
+                    { "cmd" : cmd,
+                      "iface" : self.iface,
+                      "hostname" : hostname,
+                      'dhclientconf' : dhclient_conf_path }
         elif flavor == "release":
             return client_dict[client_name]['release'] % {"cmd":cmd, "iface":self.iface}
         else:
@@ -511,7 +564,7 @@ class BaseInterface(object):
 
         while not udhcpc_complete:
             line = pipe.readline()
-            if line.endswith("failing"):
+            if line.endswith("failing."):
                 udhcpc_success = False
                 udhcpc_complete = True
             elif line == '':
@@ -523,7 +576,7 @@ class BaseInterface(object):
     def _check_dhcp_result(self, success):
         """ Print and return the correct DHCP connection result. 
         
-        Keyword Arguents:
+        Keyword Arguments:
         success -- boolean specifying if DHCP was succesful.
         
         Returns:
@@ -538,31 +591,36 @@ class BaseInterface(object):
             return 'dhcp_failed'
             
     @neediface(False)
-    def StartDHCP(self):
+    def StartDHCP(self, hostname):
         """ Start the DHCP client to obtain an IP address.
+
+        Keyword Arguments:
+        hostname -- the hostname to send to the DHCP server
         
         Returns:
         A string representing the result of the DHCP command.  See
         _check_dhcp_result for the possible values.
         
         """
-        cmd = self._get_dhcp_command('connect')
+        cmd = self._get_dhcp_command('connect', hostname)
         if self.verbose: print cmd
         self.dhcp_object = misc.Run(cmd, include_stderr=True, return_obj=True)
         pipe = self.dhcp_object.stdout
+        client_dict = { misc.DHCLIENT : self._parse_dhclient,
+                        misc.DHCPCD : self._parse_dhcpcd,
+                        misc.PUMP : self._parse_pump,
+                        misc.UDHCPC : self._parse_udhcpc,
+                      }
         
-        DHCP_CLIENT = self._get_dhcp_command() 
-        if DHCP_CLIENT == misc.DHCLIENT:
-            return self._parse_dhclient(pipe)
-        elif DHCP_CLIENT == misc.PUMP:
-            return self._parse_pump(pipe)
-        elif DHCP_CLIENT == misc.DHCPCD:
-            return self._parse_dhcpcd(pipe)
-        elif DHCP_CLIENT == misc.UDHCPC:
-            return self._parse_udhcpc(pipe)
+        DHCP_CLIENT = self._get_dhcp_command()
+        if DHCP_CLIENT in client_dict:
+            ret = client_dict[DHCP_CLIENT](pipe)
         else:
-            print 'ERROR no dhclient found!'
-    
+            print "ERROR: no dhcp client found"
+            ret = None
+        self.dhcp_object.wait()
+        return ret
+        
     @neediface(False)
     def ReleaseDHCP(self):
         """ Release the DHCP lease for this interface. """
@@ -668,8 +726,12 @@ class BaseInterface(object):
             output = self.GetIfconfig()
         else:
             output = ifconfig
-        return misc.RunRegex(ip_pattern, output)
-    
+        # check multiple ifconfig output styles
+        for pat in [ip_pattern, ip_pattern1]:
+            m = misc.RunRegex(pat, output)
+            if m: return m
+        return None
+
     @neediface(False)
     def VerifyAPAssociation(self, gateway):
         """ Verify assocation with an access point. 
@@ -678,7 +740,16 @@ class BaseInterface(object):
         trying to ping it.
         
         """
-        cmd = "ping -q -w 3 -c 1 %s" % gateway
+        if "iputils" in misc.Run(["ping", "-V"]):
+            cmd = "ping -q -w 3 -c 1 %s" % gateway
+        else:
+            # ping is from inetutils-ping (which doesn't support -w)
+            # or from some other package
+            #
+            # If there's no AP association, this will wait for
+            # timeout, while the above will wait (-w) 3 seconds at
+            # most.
+            cmd = "ping -q -c 1 %s" % gateway
         if self.verbose: print cmd
         return misc.LaunchAndWait(cmd)
 
@@ -697,6 +768,13 @@ class BaseInterface(object):
             print "Could not open %s, using ifconfig to determine status" % flags_file
             return self._slow_is_up(ifconfig)
         return bool(int(flags, 16) & 1)
+    
+    @neediface(False)
+    def StopWPA(self):
+        """ Terminates wpa using wpa_cli"""
+        cmd = 'wpa_cli -i %s terminate' % self.iface
+        if self.verbose: print cmd
+        misc.Run(cmd)
         
         
     def _slow_is_up(self, ifconfig=None):
@@ -708,6 +786,11 @@ class BaseInterface(object):
         lines = output.split('\n')
         if len(lines) < 5:
             return False
+        # check alternative ifconfig output style
+        m = misc.RunRegex(ip_up, lines[0])
+        if m and m.find('UP') > -1:
+            return True
+        # check classic ifconfig output style
         for line in lines[1:4]:
             if line.strip().startswith('UP'):
                 return True   
@@ -816,6 +899,13 @@ class BaseWiredInterface(BaseInterface):
         else:
             return False
         
+    def Authenticate(self, network):
+        misc.ParseEncryption(network)
+        cmd = ['wpa_supplicant', '-B', '-i', self.iface, '-c',
+               os.path.join(wpath.networks, 'wired'),
+               '-Dwired']
+        if self.verbose: print cmd
+        misc.Run(cmd)
 
 class BaseWirelessInterface(BaseInterface):
     """ Control a wireless network interface. """
@@ -843,7 +933,7 @@ class BaseWirelessInterface(BaseInterface):
         essid -- essid to set the interface to
 
         """
-        cmd = ['iwconfig', self.iface, 'essid', essid]
+        cmd = ['iwconfig', self.iface, 'essid', '--', str(essid)]
         if self.verbose: print str(cmd)
         misc.Run(cmd)
 
@@ -1030,9 +1120,7 @@ class BaseWirelessInterface(BaseInterface):
         bssid -- bssid of the network
 
         """
-        cmd = ['iwconfig', self.iface, 'essid', essid]
-        if self.verbose: print str(cmd)
-        misc.Run(cmd)
+        self.SetEssid(essid)
         base = "iwconfig %s" % self.iface
         if channel and str(channel).isdigit():
             cmd = "%s channel %s" % (base, str(channel))
@@ -1054,7 +1142,7 @@ class BaseWirelessInterface(BaseInterface):
         if not wpa_pass_path: return None
         key_pattern = re.compile('network={.*?\spsk=(.*?)\n}.*',
                                  re.I | re.M  | re.S)
-        cmd = [wpa_pass_path, network['essid'], str(network['key'])]
+        cmd = [wpa_pass_path, str(network['essid']), str(network['key'])]
         if self.verbose: print cmd
         return misc.RunRegex(key_pattern, misc.Run(cmd))
 
@@ -1177,7 +1265,13 @@ class BaseWirelessInterface(BaseInterface):
         except (UnicodeDecodeError, UnicodeEncodeError):
             print 'Unicode problem with current network essid, ignoring!!'
             return None
-        if ap['essid'] in ['<hidden>', "", None]:
+
+        # We (well, DBus) don't support ESSIDs with null bytes in it.
+        # From some bugreports, it seems like some APs transmit the "hidden"
+        # essid as NULL bytes. Let's strip them off.
+        ap['essid'] = ap['essid'].replace('\x00', '')
+
+        if ap['essid'] in ['Hidden', '<hidden>', "", None]:
             print 'hidden'
             ap['hidden'] = True
             ap['essid'] = "<hidden>"
@@ -1200,6 +1294,9 @@ class BaseWirelessInterface(BaseInterface):
 
         # Mode
         ap['mode'] = misc.RunRegex(mode_pattern, cell)
+        if ap['mode'] is None:
+            print 'Invalid network mode string, ignoring!'
+            return None
 
         # Break off here if we're using a ralink card
         if self.wpa_driver == RALINK_DRIVER:
@@ -1302,13 +1399,6 @@ class BaseWirelessInterface(BaseInterface):
         print 'wpa_supplicant rescan forced...'
         cmd = 'wpa_cli -i' + self.iface + ' scan'
         misc.Run(cmd)
-        
-    @neediface(False)
-    def StopWPA(self):
-        """ Terminates wpa using wpa_cli"""
-        cmd = 'wpa_cli -i %s terminate' % self.iface
-        if self.verbose: print cmd
-        misc.Run(cmd)
 
     @neediface("")
     def GetBSSID(self, iwconfig=None):
@@ -1408,8 +1498,6 @@ class BaseWirelessInterface(BaseInterface):
             output = self.GetIwconfig()
         else:
             output = iwconfig
-        signaldbm_pattern = re.compile('.*Signal level:?=? ?(-\d\d*)',
-                                       re.I | re.M | re.S)
         dbm_strength = misc.RunRegex(signaldbm_pattern, output)
         return dbm_strength
 
@@ -1425,8 +1513,7 @@ class BaseWirelessInterface(BaseInterface):
             output = self.GetIwconfig()
         else:
             output = iwconfig
-        network = misc.RunRegex(re.compile('.*ESSID:"(.*?)"',
-                                           re.I | re.M  | re.S), output)
+        network = misc.to_unicode(misc.RunRegex(essid_pattern, output))
         if network:
             network = misc.to_unicode(network)
         return network
